@@ -9,8 +9,6 @@ import json
 import logging
 import mimetypes
 import os
-import platform
-import sys
 import uuid
 import warnings
 from http.client import HTTPResponse
@@ -25,9 +23,10 @@ import aiohttp
 from aiohttp import FormData, BasicAuth
 
 import slack.errors as err
-import slack.version as slack_version
 from slack.errors import SlackRequestError
-from slack.web import convert_bool_to_0_or_1
+from slack.web import convert_bool_to_0_or_1, get_user_agent
+from slack.web.classes.attachments import Attachment
+from slack.web.classes.blocks import Block
 from slack.web.slack_response import SlackResponse
 
 
@@ -69,7 +68,12 @@ class BaseClient:
             return loop
 
     def _get_headers(
-        self, has_json: bool, has_files: bool, request_specific_headers: Optional[dict]
+        self,
+        *,
+        token: Optional[str],
+        has_json: bool,
+        has_files: bool,
+        request_specific_headers: Optional[dict],
     ):
         """Constructs the headers need for a request.
         Args:
@@ -86,12 +90,12 @@ class BaseClient:
                 }
         """
         final_headers = {
-            "User-Agent": self._get_user_agent(),
+            "User-Agent": get_user_agent(),
             "Content-Type": "application/x-www-form-urlencoded",
         }
 
-        if self.token:
-            final_headers.update({"Authorization": "Bearer {}".format(self.token)})
+        if token:
+            final_headers.update({"Authorization": "Bearer {}".format(token)})
 
         # Merge headers specified at client initialization.
         final_headers.update(self.headers)
@@ -169,8 +173,18 @@ class BaseClient:
         if params is not None and isinstance(params, dict):
             params = {k: v for k, v in params.items() if v is not None}
 
+        token: Optional[str] = self.token
+        if params is not None and "token" in params:
+            token = params.pop("token")
+        if json is not None and "token" in json:
+            token = json.pop("token")
         req_args = {
-            "headers": self._get_headers(has_json, has_files, headers),
+            "headers": self._get_headers(
+                token=token,
+                has_json=has_json,
+                has_files=has_files,
+                request_specific_headers=headers,
+            ),
             "data": data,
             "files": files,
             "params": params,
@@ -564,7 +578,7 @@ class BaseClient:
         self, token: str, has_json: bool, has_files: bool, additional_headers: dict,
     ):
         headers = {
-            "User-Agent": self._get_user_agent(),
+            "User-Agent": get_user_agent(),
             "Content-Type": "application/x-www-form-urlencoded",
         }
         headers.update(self.headers)
@@ -581,24 +595,6 @@ class BaseClient:
 
     # =================================================================
 
-    @staticmethod
-    def _get_user_agent():
-        """Construct the user-agent header with the package info,
-        Python version and OS version.
-
-        Returns:
-            The user agent string.
-            e.g. 'Python/3.6.7 slackclient/2.0.0 Darwin/17.7.0'
-        """
-        # __name__ returns all classes, we only want the client
-        client = "{0}/{1}".format("slackclient", slack_version.__version__)
-        python_version = "Python/{v.major}.{v.minor}.{v.micro}".format(
-            v=sys.version_info
-        )
-        system_info = "{0}/{1}".format(platform.system(), platform.release())
-        user_agent_string = " ".join([python_version, client, system_info])
-        return user_agent_string
-
     @staticmethod
     def validate_slack_signature(
         *, signing_secret: str, data: str, timestamp: str, signature: str
@@ -636,6 +632,37 @@ class BaseClient:
         calculated_signature = f"v0={request_hash}"
         return hmac.compare_digest(calculated_signature, signature)
 
+    @staticmethod
+    def _parse_web_class_objects(kwargs) -> None:
+        def to_dict(obj: Union[Dict, Block, Attachment]):
+            if isinstance(obj, Block):
+                return obj.to_dict()
+            if isinstance(obj, Attachment):
+                return obj.to_dict()
+            return obj
+
+        blocks = kwargs.get("blocks", None)
+        if blocks is not None and isinstance(blocks, list):
+            dict_blocks = [to_dict(b) for b in blocks]
+            kwargs.update({"blocks": dict_blocks})
+
+        attachments = kwargs.get("attachments", None)
+        if attachments is not None and isinstance(attachments, list):
+            dict_attachments = [to_dict(a) for a in attachments]
+            kwargs.update({"attachments": dict_attachments})
+
+    @staticmethod
+    def _update_call_participants(kwargs, users: Union[str, List[Dict[str, str]]]):
+        if users is None:
+            return
+
+        if isinstance(users, list):
+            kwargs.update({"users": json.dumps(users)})
+        elif isinstance(users, str):
+            kwargs.update({"users": users})
+        else:
+            raise SlackRequestError("users must be either str or List[Dict[str, str]]")
+
 
 # https://api.slack.com/changelog/2020-01-deprecating-antecedents-to-the-conversations-api
 deprecated_method_prefixes_2020_01 = ["channels.", "groups.", "im.", "mpim."]
